//
//  CanvasLayer.m
//  npapiHelloWorld
//
//  Created by Tomas Rycl on 11.09.14.
//  Copyright (c) 2014 Tomas Rycl. All rights reserved.
//

#import "CanvasLayer.h"

static CGImageRef CreateCGImageFromBuffer(CFDataRef buffer, CGSize size, CGImageAlphaInfo alphaInfo)
{
    size_t bitsPerPixel = 8;
    size_t components = 4;

    CGDataProviderRef dataProvider = CGDataProviderCreateWithCFData(buffer);

    CGImageRef result = CGImageCreate((size_t)size.width, (size_t)size.height,
                                      bitsPerPixel, bitsPerPixel * components,
                                      (size_t)size.width * components,
                                      CGColorSpaceCreateDeviceRGB(),
                                      kCGBitmapByteOrder32Little,
                                      dataProvider, NULL, FALSE,
                                      kCGRenderingIntentDefault);

    CFRelease(dataProvider); dataProvider = NULL;

    return result;
}

static CGImageRef CreateCGImageWithSize(const CGSize size, bool mirror)
{
    size_t bufferBytesSize = size.width * size.height * sizeof(uint32_t);

    CFMutableDataRef bufferContainer = CFDataCreateMutable(kCFAllocatorDefault, bufferBytesSize);

    uint32_t *buffer = (uint32_t *)CFDataGetMutableBytePtr(bufferContainer);

    for (int y = 0; y < size.height; ++y)
    {
        for (int x = 0; x < size.width; ++x)
        {
            buffer[(size_t)((y * size.width) + x)] = (((/*x +*/ y+1) % 4) != 0) ? 0xffffffff : 0x00000000;
        }
    }

    CGImageRef result =  CreateCGImageFromBuffer(bufferContainer, size, kCGImageAlphaNoneSkipFirst);

    CFRelease(bufferContainer); bufferContainer = NULL;

    return result;
}

static void DrawHorizontalLinesInContext(const CGSize size, CGContextRef context, const CGFloat scale)
{
    //Set the stroke (pen) color
    CGContextSetStrokeColorWithColor(context, CGColorGetConstantColor(kCGColorBlack));

    //Set the width of the pen mark
    CGContextSetLineWidth(context, 1.0 / scale);

    CGFloat width = size.width;

    //(scale - 1.0) * 0.5
    for (CGFloat y = 0.0 ; y < size.height; y += (2.0 / scale))
    {
        CGContextMoveToPoint(context, 0.0, y);

        CGContextAddLineToPoint(context, width, y);

        CGContextStrokePath(context);
    }
}

static void DrawImageWithHorizontalLinesInContext(const CGSize imageSize, const CGSize rectSize, CGContextRef context)
{
    NSLog(@"Image render size width:%.0f, height:%.0f",
          imageSize.width, imageSize.height);

    CGImageRef image = CreateCGImageWithSize(imageSize, FALSE);

    CGRect imageRect = CGRectMake(0.0, 0.0, rectSize.width, rectSize.height);

    CGContextDrawImage(context, imageRect, image);

    CFRelease(image); image = NULL;
}

@implementation CanvasLayer

- (id)init
{
    if ((self = [super init]) == nil) return nil;

    //Initialization

    self.needsDisplayOnBoundsChange = YES;
    self.frame = CGRectMake(0, 0, 0, 0);
    self.autoresizingMask = kCALayerWidthSizable | kCALayerHeightSizable;

    return self;
}

- (void)drawInContext:(CGContextRef)context
{
    CGContextSaveGState(context);

    size_t w = CGBitmapContextGetWidth(context);
    size_t h = CGBitmapContextGetHeight(context);

    NSLog(@"Bitmap context, bounds width:%zu, height:%zu", w, h);

    NSLog(@"Layer, bounds width:%.0f, height:%.0f",
          self.bounds.size.width, self.bounds.size.height);

    CGSize imageUse = CGSizeMake(self.bounds.size.width * self.contentsScale,
                                 self.bounds.size.height * self.contentsScale);
    DrawImageWithHorizontalLinesInContext(imageUse, self.bounds.size, context);

    //DrawHorizontalLinesInContext(self.bounds.size, context, self.contentsScale);

    //CGContextSetFillColorWithColor(context, CGColorGetConstantColor(kCGColorBlack));
    //CGContextFillRect(context, self.bounds);

    CGContextRestoreGState(context);
}

@end
