#import "CanvasNPObject.h"

CanvasNPObject::CanvasNPObject()
{
    static bool identifiersInitialized = false;
    if (!identifiersInitialized)
    {
        // Take all method identifier names and convert them to NPIdentifiers.
        browser->getstringidentifiers(methodIdentifierNames, NUM_METHOD_IDENTIFIERS, methodIdentifiers);
        identifiersInitialized = true;
    }
}

bool CanvasNPObject::performMethodByName(NPIdentifier name)
{
    if (name == methodIdentifiers[ID_SHOW]) {
        this->show();
        return true;
    }

    if (name == methodIdentifiers[ID_HIDE]) {
        this->hide();
        return true;
    }

    return false;
}

void CanvasNPObject::show() {};

void CanvasNPObject::hide() {};

//////////////////////////////////////////////////////////////////////////////
// CanvasNPObject structure

static NPObject *CanvasNPObjectAllocate(NPP npp, NPClass* theClass)
{
    return new CanvasNPObject;
}

static void CanvasNPObjectDeallocate(NPObject *npObject)
{
    delete npObject;
}

static bool CanvasNPObjectHasMethod(NPObject *obj, NPIdentifier name)
{
    // Loop over all the method NPIdentifiers and see if we expose the given method.
    for (int i = 0; i < NUM_METHOD_IDENTIFIERS; i++)
    {
        if (name == methodIdentifiers[i]) return true;
    }

    return false;
}

static bool CanvasNPObjectInvoke(NPObject *npObject, NPIdentifier name, const NPVariant* args, uint32_t argCount, NPVariant* result)
{
    CanvasNPObject *obj = static_cast<CanvasNPObject *>(npObject);

    return obj->performMethodByName(name);
}

//////////////////////////////////////////////////////////////////////////////

static NPClass CanvasNPClass =
{
    NP_CLASS_STRUCT_VERSION,
    CanvasNPObjectAllocate,   // NP_Allocate
    CanvasNPObjectDeallocate, // NP_Deallocate
    0,                        // NP_Invalidate
    CanvasNPObjectHasMethod,  // NP_HasMethod
    CanvasNPObjectInvoke,     // NP_Invoke
    0,                        // NP_InvokeDefault
    0,                        // NP_HasProperty
    0,                        // NP_GetProperty
    0,                        // NP_SetProperty
    0,                        // NP_RemoveProperty
    0,                        // NP_Enumerate
    0,                        // NP_Construct
};

//////////////////////////////////////////////////////////////////////////////

NPObject *CreateCanvasNPObject(NPNetscapeFuncs* browser, NPP npp)
{
    CanvasNPObject *canvasNPObject = (CanvasNPObject *)browser->createobject(npp, &CanvasNPClass);

    return canvasNPObject;
}
