
#include "PluginInstance.h"
#include "BrowserAPI.h"
#include "CanvasNPObject.h"
#include "CanvasLayer.h"

#include "PluginAPI.h"

NPError NPP_New(NPMIMEType pluginType, NPP instance, uint16_t mode, int16_t argc, char* argn[], char* argv[], NPSavedData* saved)
{
//    return NPERR_INVALID_URL;

    // Create per-instance storage
    PluginInstance *obj = new PluginInstance;

    obj->npp = instance;
    instance->pdata = obj;

    //////////////////////////////////////////////////////////////////////////

    // Ask the browser if it supports the Core Animation drawing model
    NPBool supportsCoreAnimation;
    if (browser->getvalue(instance, NPNVsupportsCoreAnimationBool, &supportsCoreAnimation) != NPERR_NO_ERROR)
        supportsCoreAnimation = FALSE;

    if (!supportsCoreAnimation)
        return NPERR_INCOMPATIBLE_VERSION_ERROR;

    // If the browser supports the Core Animation drawing model, enable it.
    browser->setvalue(instance, NPPVpluginDrawingModel, (void *)NPDrawingModelCoreAnimation);

    //////////////////////////////////////////////////////////////////////////

    // If the browser supports the Cocoa event model, enable it.
    NPBool supportsCocoa;
    if (browser->getvalue(instance, NPNVsupportsCocoaBool, &supportsCocoa) != NPERR_NO_ERROR)
        supportsCocoa = FALSE;

    if (!supportsCocoa)
        return NPERR_INCOMPATIBLE_VERSION_ERROR;

    browser->setvalue(instance, NPPVpluginEventModel, (void *)NPEventModelCocoa);

    //////////////////////////////////////////////////////////////////////////

    return NPERR_NO_ERROR;
}

NPError NPP_Destroy(NPP instance, NPSavedData** save)
{
    PluginInstance *obj = static_cast<PluginInstance *>(instance->pdata);

    delete obj;

    obj = nullptr;
    instance->pdata = nullptr;

    return NPERR_NO_ERROR;
}

NPError NPP_SetWindow(NPP instance, NPWindow* window)
{
    PluginInstance *obj = static_cast<PluginInstance *>(instance->pdata);
    obj->window = *window;

    NSLog(@"NPWindow platform specific window is: %p", window->window);
    NSLog(@"NPWindow width: %u, height: %u", window->width, window->height);
    NSLog(@"NPWindow clipping, top: %u, bottom: %u, left: %u, right :%u",
          window->clipRect.top, window->clipRect.bottom,
          window->clipRect.left, window->clipRect.right);

    //This works in Firefox which set up Window every time it moves
    //from retina to non-retina. It does not work in Safari and Chrome which
    //set up Window only once.
    //Reading and setting value here will ensure proper behavior on retina
    //with Firefox.

    double browserScale = 1.0;

    browser->getvalue(instance, NPNVcontentsScaleFactor, &browserScale);
    NSLog(@"Scale reported from browser is: %.0f", browserScale);

    ////

    obj->windowScale = browserScale;
    obj->rootLayer.contentsScale = obj->windowScale;

    return NPERR_NO_ERROR;
}

NPError NPP_GetValue(NPP instance, NPPVariable variable, void *value)
{
    PluginInstance *obj = static_cast<PluginInstance *>(instance->pdata);

    switch (variable)
    {
        case NPPVpluginCoreAnimationLayer:
        {
            if (!obj->rootLayer)
            {
                obj->rootLayer = [[CanvasLayer alloc] init];

                obj->rootLayer.contentsScale = obj->windowScale;
            }

            *(CALayer **)value = obj->rootLayer;

            return NPERR_NO_ERROR;
        }

        case NPPVpluginScriptableNPObject:
        {
            if (!obj->canvasNPObject)
                obj->canvasNPObject = CreateCanvasNPObject(browser, instance);

            // The NPAPI standard specifies that a retained NPObject should be returned.
            *(NPObject **)value = obj->canvasNPObject;
            browser->retainobject(obj->canvasNPObject);

            return NPERR_NO_ERROR;
        }

        default:
            return NPERR_GENERIC_ERROR;
    }
}

NPError NPP_SetValue(NPP instance, NPNVariable variable, void *value)
{
    //Safari reports new scale factor using NPP_SetValue
    //Firefox and Chrome do not do this

    //Chrome just never call this callback - this is never logged in chrome
    NSLog(@"Browser announced NPNVariable: %d", variable);

    if (variable == NPNVcontentsScaleFactor)
    {
        double browserScale = 1.0;
        if (nullptr != value)
        {
            browserScale = *(reinterpret_cast<double *>(value));
            NSLog(@"Browser announced a new scale: %.0f", browserScale);

            PluginInstance *obj = static_cast<PluginInstance *>(instance->pdata);

            obj->windowScale = browserScale;
            obj->rootLayer.contentsScale = obj->windowScale;
            [obj->rootLayer setNeedsDisplay];
        }

        return NPERR_NO_ERROR;
    }

    return NPERR_GENERIC_ERROR;
}
